
// GunnerCanvas.java
// Andrew Davison, dandrew@fivedots.coe.psu.ac.th, November 2004

/* Create a 3D scene using retained mode (i.e. a scenegraph).
   
   The scene contains NUM_PENGUINS penguins on a randomly chosen tiled floor,
   and a randomly chosen image background. The location of the penguins
   is random as well, but somewhere on the floor.

   The scene has one directional light.

   The user can move around the scene firing a gun at the penguins,
   with accompanying visual and audio feedback. Camera movement
   is handled by MobileGunCamera, and shooting is dealt with by
   ShotManager.

   The scene is updated and repainted by a timer task in 
   GunnerM3G which calls update() periodically.
*/

import java.util.*;

import javax.microedition.lcdui.*;
import javax.microedition.m3g.*;


public class GunnerCanvas extends Canvas implements CommandListener
{
  private static final int FLOOR_LEN = 8;   // should be even
  private static final int NUM_PENGUINS = 2;

  private Command exitCmd; 

  private Graphics3D iG3D;

  private GunnerM3G top;
  private World scene;
  private MobileGunCamera mobCam;
  private ShotManager shotMng;

  private Image hitIm;    // hit image
  private int xHit;       // x-coord of hit image on screen

  private Random rnd;     // for selecting the background and floor images


  public GunnerCanvas(GunnerM3G top) 
  { 
    this.top = top;

    exitCmd = new Command("Exit", Command.EXIT, 0);
    addCommand(exitCmd);
    setCommandListener(this);

    loadHitImage(); 
    rnd = new Random();

    iG3D = Graphics3D.getInstance();
    scene = new World();   // create scene graph
    buildScene();
  }  // end of GunnerCanvas()


  public void commandAction(Command command, Displayable disp)
  { 
    if (command == exitCmd) { // finish
      shotMng.closeSounds();
      top.finishGame();
    }
    else   // execution should not get here
      System.out.println("Command Action Error: " + command);
  } // end of commandAction()


  private void buildScene()
  // add elements to the scene
  {
    addCamera();
    addLights();
    addBackground();
    addFloor();
    addPenguins();
    shotMng = new ShotManager(scene, mobCam); // add explosions stuff
  } // end of buildScene()


  private void addCamera()
  // the camera is a MobileGunCamera object
  {
    mobCam = new MobileGunCamera( getWidth(), getHeight());
    scene.addChild( mobCam.getCameraGroup() );
    scene.setActiveCamera( mobCam.getCamera() ); 
  }  // end of addCamera()


  private void addLights()
  {
    Light light = new Light();  // default white, directional light
    light.setIntensity(1.25f);  // make it a bit brighter
    light.setOrientation(-45.0f, 1.0f, 0, 0); // pointing down and into scene
    scene.addChild(light);
  }  // end of addLights()


  private void addBackground()
  /* Use a background image. The choice of image
     is made at random from 3 possibilities.
  */
  { Background backGnd = new Background();
    Image2D backIm = null;

    int choice = rnd.nextInt(3);
    System.out.println("Adding background image " + choice);

    switch (choice) {
      case 0: backIm = loadImage("/sky/evening.gif"); break;
      case 1: backIm = loadImage("/sky/orangy.gif"); break;
      case 2: backIm = loadImage("/sky/purply.gif"); break;
      default: System.out.println("No background image"); break;
    }

    if (backIm != null) 
      backGnd.setImage( backIm );
    else 
      backGnd.setColor(0x00bffe); // a light blue background

    scene.setBackground(backGnd);
  } // end of addBackground()


  private void addFloor()
  /* Tile an image over an FLOOR_LEN-by-FLOOR_LEN sized floor. 
     The choice of image is made at random from 4 possibilities.
  */
  { Image2D floorIm = null;

    int choice = rnd.nextInt(4);
    System.out.println("Adding floor image " + choice);

    switch (choice) {
      case 0: floorIm = loadImage("/floor/grass.gif"); break;
      case 1: floorIm = loadImage("/floor/stone.jpg"); break;
      case 2: floorIm = loadImage("/floor/cobbles.jpg"); break;
      case 3: floorIm = loadImage("/floor/plate.jpg"); break;
      default: System.out.println("No floor image"); break;
    }

    TiledFloor f = new TiledFloor( floorIm, FLOOR_LEN);   
      // FLOOR_LEN by FLOOR_LEN size, made up of 1 by 1 tiles

    scene.addChild( f.getFloorMesh() );
  }  // end of addFloor()


  private void addPenguins()
  // add penguins at random locations (don't check for overlapping)
  {
    PenguinModel pm;
    float x, z;
    for (int i=0; i < NUM_PENGUINS; i++) {
      x = rndCoord(); z = rndCoord();
      System.out.println("Adding Penguin " + i + " at (" +
                                           x + ", " + z + ") ...");
      pm = new PenguinModel("Penguin " + i, i, x, z);
      scene.addChild( pm.getModel() );   // add the model
    }
  }  // end of addPenguins()


  private float rndCoord()
  // generate a coordinate in the range -FLOOR_LEN/2 to FLOOR_LEN/2
  {
    float f = rnd.nextFloat();  // 0.0f to 1.0f
    float coord = (f* FLOOR_LEN) - FLOOR_LEN/2.0f;
    return coord;
  } // end of rndCoord()


  // ---------------image loading -----------------------------


  private Image2D loadImage(String fn)
  // load the image stored in fn
  { Image2D im = null;
    try {
       im = (Image2D)Loader.load(fn)[0];
    }
    catch (Exception e)
    { System.out.println("Cannot make image from " + fn); }
    return im;
  }  // end of loadImage()


  private void loadHitImage()
  /* Load the hit message as a Image object, and calculate its
     drawing position. */
  {
    hitIm = null;
    xHit = 0;
    try {
      hitIm = Image.createImage("/hit.gif");
    }
    catch (Exception e)
    { System.out.println("Hit image not found"); }

    /* Calculate the x-coordinate for the image so it can be drawn in
       the middle of the screen on the x-axis. */
    if (hitIm != null)
      xHit = (getWidth()/2) - (hitIm.getWidth()/2);
  } // end of loadHitImage;


  // ------------ process key presses and releases ------------

  protected void keyPressed(int keyCode)
  { 
    int gameAction = getGameAction(keyCode);
    if (gameAction == Canvas.FIRE)
      shotMng.shootGun();
    else
      mobCam.pressedKey(gameAction);
  } // end of keyPressed()


  protected void keyReleased(int keyCode)
  { int gameAction = getGameAction(keyCode);
    mobCam.releasedKey(gameAction);
  }


  // ------------------ update and paint the canvas ------------------

  public void update()
  // called by the timer task in GunnerM3G
  {
    mobCam.update();    // update the camera
    shotMng.update();   // update the shot manager

    repaint();
  }  // end of update()


  protected void paint(Graphics g)
  {
    iG3D.bindTarget(g);
    try {
      iG3D.render(scene);
    }
    catch(Exception e)
    { e.printStackTrace();  }
    finally {
      iG3D.releaseTarget();
    }

    // maybe show a hit message (an image and text)
    if ( shotMng.hitMade() ) {
      g.drawImage( hitIm, xHit, 10, Graphics.TOP|Graphics.LEFT);

      // large and bold
	  g.setFont(Font.getFont(Font.FACE_PROPORTIONAL, 
                             Font.STYLE_BOLD, Font.SIZE_LARGE));
      g.setColor(0xFF0000);  // red
      g.drawString( "ID of penguin: " + shotMng.getHitID(), 5,5, 
                                      Graphics.TOP|Graphics.LEFT);
    }
  } // end of paint()

}  // end of GunnerCanvas class
